@@ -8,8 +8,9 @@
  */
 
 package org.gnu.atk;
-import org.gnu.glib.Boxed;
-public class State extends Boxed 
+
+
+public class State
 {
     public static String getTypeName(StateType type) {
         return atk_state_type_get_name(type.getValue());
