@@ -8,7 +8,9 @@
  */
 
 package org.gnu.atk;
-import org.gnu.glib.Enum;
+
+import org.gnu.javagnome.Enum;
+
 public class StateType extends Enum 
 {
 
