@@ -8,80 +8,91 @@
  */
 
 package org.gnu.gdk;
+
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.pango.Font;
 import org.gnu.pango.GlyphString;
 import org.gnu.pango.Layout;
 import org.gnu.pango.LayoutLine;
+import org.gnu.javagnome.Handle;
 
 /**
- * Provides support for drawing points, lines, arcs, and text onto what we
- * call 'drawables'.  Drawables, as the name suggests, are things which support
- * drawing onto them and are either org.gnu.gdk.Window or org.gnu.gdk.Pixmap
- * objects.
+ * Provides support for drawing points, lines, arcs, and text onto
+ * what we call 'drawables'.  Drawables, as the name suggests, are
+ * things which support drawing onto them and are either
+ * org.gnu.gdk.Window or org.gnu.gdk.Pixmap objects.
  */
  
 public class Drawable extends GObject 
 {
 	
-	public Drawable(Handle handle){
-		super(handle);
-	}
+    Drawable(Handle handle){
+        super(handle);
+    }
 	
-	/**
-     *  Returns the size of the Drawable. 
-	 */
+    /**
+     *  Returns the size of the Drawable.
+     *  @deprecated use getWidth and getHeight instead. 
+     */
     public Dimension getSize(){
-		int width[]=new int[1];
-		int height[]=new int[1];
-		Drawable.gdk_drawable_get_size(getHandle(),width,height);
-		return new Dimension(width[0],height[0]);
-	}
+        int width[]=new int[1];
+        int height[]=new int[1];
+        Drawable.gdk_drawable_get_size(getHandle(),width,height);
+        return new Dimension(width[0],height[0]);
+    }
 	
-	/**
+    /**
+     * Return the width of the Drawable
+     */
+    public int getWidth() {
+        int width[]=new int[1];
+        int height[]=new int[1];
+        Drawable.gdk_drawable_get_size(getHandle(),width,height);
+        return width[0];
+    }
+	
+    /**
+     * Return the height of the Drawable.
+     */
+    public int getHeight() {
+        int width[]=new int[1];
+        int height[]=new int[1];
+        Drawable.gdk_drawable_get_size(getHandle(),width,height);
+        return height[0];
+    }
+	
+    /**
      * Sets the Colormap associated with the Drawable.  Normally this happens
      * automatically when the Drawable is created.   
      * @param cmap
-	 */
+     */
     public void setColormap(Colormap cmap){
-		Drawable.gdk_drawable_set_colormap(getHandle(),cmap.getHandle());
-	}
+        Drawable.gdk_drawable_set_colormap(getHandle(),cmap.getHandle());
+    }
 	
     /**
      * Gets the Colormap for the Drawable if one is set; returns null
      * otherwise.
      */
-	public Colormap getColormap(){
-	    Handle hndl = gdk_drawable_get_colormap(getHandle());
-        if (null == hndl)
-            return null;
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Colormap)obj;
-		return new Colormap(hndl);
-	}
+    public Colormap getColormap(){
+        return Colormap.getColormapFromHandle( gdk_drawable_get_colormap(getHandle()) );
+    }
 	
-	/**
+    /**
      * Gets the Visual describing the pixel format of this Drawable. 
-	 */
+     */
     public Visual getVisual(){
-	    Handle hndl = gdk_drawable_get_visual(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Visual)obj;
-		return new Visual(hndl);
-	}
+        return Visual.getVisualFromHandle( gdk_drawable_get_visual(getHandle()) );
+    }
 	
-	
-	/**
+    /**
      * Obtains the bit depth of the Drawable, this is, the number
      * of bits that make up a pixel. 
-	 */
+     */
     public int getDepth(){
-		return Drawable.gdk_drawable_get_depth(getHandle());
-	}
+        return Drawable.gdk_drawable_get_depth(getHandle());
+    }
 	
     /**
      * Obtains the pixels from a server-side Drawable as a client-side Image.
@@ -92,92 +103,93 @@ public class Drawable extends GObject
      * @param width
      * @param height
      */
-	public Image getImage(int x, int y, int width, int height){
-	    Handle hndl = gdk_drawable_get_image(getHandle(),x,y,width,height);
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Image)obj;
-		return new Image(hndl);
-	}
+    public Image getImage(int x, int y, int width, int height){
+        Handle hndl = gdk_drawable_get_image(getHandle(),x,y,width,height);
+        if ( hndl != null ) {
+            GObject obj = getGObjectFromHandle(hndl);
+            return (obj != null) ? (Image)obj : new Image(hndl);
+        }
+        return null;
+    }
 	
-	/**
+    /**
      * Computes the Region of a Drawable that potentially can be written to
      * by drawing primitives.  This Region will not take into account the
      * clip region for the GC, and may also not take into account other factors
      * such as if the Window is obscured by other Windows.   
-	 */
+     */
     public Region getClipRegion(){
-		return new Region(Drawable.gdk_drawable_get_clip_region(getHandle()));
-	}
+        return Region.getRegionFromHandle(Drawable.gdk_drawable_get_clip_region(getHandle()));
+    }
 	
     /**
      * Computes the Region of a Drawable that is potentially visible.
      */
-	public Region getVisibleRegion(){
-		return new Region(Drawable.gdk_drawable_get_visible_region(getHandle()));
-	}
+    public Region getVisibleRegion(){
+        return Region.getRegionFromHandle(Drawable.gdk_drawable_get_visible_region(getHandle()));
+    }
 	
-	public void drawRGBImage(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
-		gdk_draw_rgb_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride);
-	}
+    public void drawRGBImage(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
+        gdk_draw_rgb_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride);
+    }
 
-	public void drawRGBImage(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
-		drawRGBImage(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride);
-	}
+    public void drawRGBImage(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
+        drawRGBImage(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride);
+    }
 
-	public void drawRGBImageDitherAlign(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
-		gdk_draw_rgb_image_dithalign(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride, xdith, ydith);
-	}
+    public void drawRGBImageDitherAlign(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
+        gdk_draw_rgb_image_dithalign(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride, xdith, ydith);
+    }
 
-	public void drawRGBImageDitherAlign(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
-		drawRGBImageDitherAlign(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride, xdith, ydith);
-	}
+    public void drawRGBImageDitherAlign(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
+        drawRGBImageDitherAlign(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride, xdith, ydith);
+    }
 
-	public void drawRGB32Image(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
-		gdk_draw_rgb_32_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride);
-	}
+    public void drawRGB32Image(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
+        gdk_draw_rgb_32_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride);
+    }
 
-	public void drawRGB32Image(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
-		drawRGB32Image(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride);
-	}
+    public void drawRGB32Image(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
+        drawRGB32Image(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride);
+    }
 
-	public void drawRGB32ImageDitherAlign(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
-		gdk_draw_rgb_32_image_dithalign(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride, xdith, ydith);
-	}
+    public void drawRGB32ImageDitherAlign(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
+        gdk_draw_rgb_32_image_dithalign(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride, xdith, ydith);
+    }
 
-	public void drawRGB32ImageDitherAlign(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
-		drawRGB32ImageDitherAlign(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride, xdith, ydith);
-	}
+    public void drawRGB32ImageDitherAlign(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, int xdith, int ydith) {
+        drawRGB32ImageDitherAlign(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride, xdith, ydith);
+    }
 
-	public void drawGrayImage(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
-		gdk_draw_gray_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride);
-	}
+    public void drawGrayImage(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
+        gdk_draw_gray_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride);
+    }
 
-	public void drawGrayImage(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
-		drawGrayImage(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride);
-	}
+    public void drawGrayImage(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride) {
+        drawGrayImage(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride);
+    }
 
-	public void drawIndexedImage(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, RgbCmap cmap) {
-		gdk_draw_indexed_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride, cmap.getHandle());
-	}
+    public void drawIndexedImage(GC gc, int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, RgbCmap cmap) {
+        gdk_draw_indexed_image(getHandle(), gc.getHandle(), x, y, width, height, dither.getValue(), buffer, rowStride, cmap.getHandle());
+    }
 
-	public void drawIndexedImage(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, RgbCmap cmap) {
-		drawIndexedImage(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride, cmap);
-	}
+    public void drawIndexedImage(int x, int y, int width, int height, RgbDither dither, byte[] buffer, int rowStride, RgbCmap cmap) {
+        drawIndexedImage(GC.getDefaultGC(this), x, y, width, height, dither, buffer, rowStride, cmap);
+    }
 
-	/**
+    /**
      * Draws an outlined or filled polygon. 
      * @param gc
      * @param filled
      * @param points
-	 */
+     */
     public void drawPolygon(GC gc, boolean filled, Point[] points) {
-	    Handle[] hndls = new Handle[points.length];
-		for (int i = 0; i < points.length; i++) {
-			hndls[i] = points[i].getHandle();
-		}
-		gdk_draw_polygon(getHandle(), gc.getHandle(), filled, hndls);
-	}
+        Handle[] hndls = new Handle[points.length];
+        for (int i = 0; i < points.length; i++) {
+            hndls[i] = points[i].getHandle();
+        }
+        gdk_draw_polygon(getHandle(), gc.getHandle(), filled, hndls);
+    }
 	
     /**
      * Convience method that draws an outlined or filled polygon
@@ -187,7 +199,7 @@ public class Drawable extends GObject
      */
     public void drawPolygon(boolean filled, Point[] points) {
     	drawPolygon(GC.getDefaultGC(this), filled, points);
-	}
+    }
 	
     /**
      * Copies the width x height region of the Drawable at coordinates
@@ -201,9 +213,9 @@ public class Drawable extends GObject
      * @param width
      * @param height
      */
-	public void drawDrawable(GC gc, Drawable src, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
-		gdk_draw_drawable(getHandle(), gc.getHandle(), src.getHandle(), xsrc, ysrc, xdest, ydest, width, height);
-	}
+    public void drawDrawable(GC gc, Drawable src, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
+        gdk_draw_drawable(getHandle(), gc.getHandle(), src.getHandle(), xsrc, ysrc, xdest, ydest, width, height);
+    }
 	
     /**
      * Convience method that copies the width x height region of 
@@ -217,9 +229,9 @@ public class Drawable extends GObject
      * @param width
      * @param height
      */
-	public void drawDrawable(Drawable src, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
-		drawDrawable(GC.getDefaultGC(this), src, xsrc, ysrc, xdest, ydest, width, height);
-	}
+    public void drawDrawable(Drawable src, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
+        drawDrawable(GC.getDefaultGC(this), src, xsrc, ysrc, xdest, ydest, width, height);
+    }
 	
     /**
      * Draw an Image onto this Drawable.  The depth of the Image must
@@ -233,9 +245,9 @@ public class Drawable extends GObject
      * @param width
      * @param height
      */
-	public void drawImage(GC gc, Image image, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
-		gdk_draw_image(getHandle(), gc.getHandle(), image.getHandle(), xsrc, ysrc, xdest, ydest, width, height);
-	}
+    public void drawImage(GC gc, Image image, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
+        gdk_draw_image(getHandle(), gc.getHandle(), image.getHandle(), xsrc, ysrc, xdest, ydest, width, height);
+    }
 	
     /**
      * Draw an Image onto this Drawable using the default GC.  
@@ -248,81 +260,81 @@ public class Drawable extends GObject
      * @param width
      * @param height
      */
-	public void drawImage(Image image, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
-		drawImage(GC.getDefaultGC(this), image, xsrc, ysrc, xdest, ydest, width, height);
-	}
+    public void drawImage(Image image, int xsrc, int ysrc, int xdest, int ydest, int width, int height) {
+        drawImage(GC.getDefaultGC(this), image, xsrc, ysrc, xdest, ydest, width, height);
+    }
 	
-	/**
+    /**
      * Draw a number of points using the forground color and other
      * attributes of the GC. 
      * @param gc
      * @param points
-	 */
+     */
     public void drawPoints(GC gc, Point[] points) {
-	    Handle[] hndls = new Handle[points.length];
-		for (int i = 0; i < points.length; i++) {
-			hndls[i] = points[i].getHandle();
-		}
-		gdk_draw_points(getHandle(), gc.getHandle(), hndls);
-	}
+        Handle[] hndls = new Handle[points.length];
+        for (int i = 0; i < points.length; i++) {
+            hndls[i] = points[i].getHandle();
+        }
+        gdk_draw_points(getHandle(), gc.getHandle(), hndls);
+    }
 
-	/**
+    /**
      * Draw a number of points using the forground color and other
      * attributes of the default GC. 
      * @param points
-	 */
+     */
     public void drawPoints(Point[] points) {
     	drawPoints(GC.getDefaultGC(this), points);
-	}
+    }
 
     /**
      * Draw a number of unconnected lines.
      * @param gc
      * @param segments
      */
-	public void drawSegments(GC gc, Segment[] segments) {
-	    Handle[] hndls = new Handle[segments.length];
-		for (int i = 0; i < segments.length; i++) {
-			hndls[i] = segments[i].getHandle();
-		}
-		gdk_draw_segments(getHandle(), gc.getHandle(), hndls);
-	}
+    public void drawSegments(GC gc, Segment[] segments) {
+        Handle[] hndls = new Handle[segments.length];
+        for (int i = 0; i < segments.length; i++) {
+            hndls[i] = segments[i].getHandle();
+        }
+        gdk_draw_segments(getHandle(), gc.getHandle(), hndls);
+    }
 	
     /**
      * Draw a number of unconnected lines using the default
      * GC.
      * @param segments
      */
-	public void drawSegments(Segment[] segments) {
-		drawSegments(GC.getDefaultGC(this), segments);
-	}
+    public void drawSegments(Segment[] segments) {
+        drawSegments(GC.getDefaultGC(this), segments);
+    }
 	
-	/**
+    /**
      * Draw a series of lines connecting the given points.  The
      * way in which joins between lines are drawn is determined
      * by the CapStyle value in GC. 
      * @param gc
      * @param points
-	 */
+     */
     public void drawLines(GC gc, Point[] points) {
-	    Handle[] hndls = new Handle[points.length];
-		for (int i = 0; i < points.length; i++) {
-			hndls[i] = points[i].getHandle();
-		}
-		gdk_draw_lines(getHandle(), gc.getHandle(), hndls);
-	}
+        Handle[] hndls = new Handle[points.length];
+        for (int i = 0; i < points.length; i++) {
+            hndls[i] = points[i].getHandle();
+        }
+        gdk_draw_lines(getHandle(), gc.getHandle(), hndls);
+    }
 
-	/**
+    /**
      * Draw a series of lines connecting the given points.  The
      * way in which joins between lines are drawn is determined
      * by the CapStyle value in the default GC. 
      * @param points
-	 */
+     */
     public void drawLines(Point[] points) {
     	drawLines(GC.getDefaultGC(this), points);
-	}
+    }
 
-	/**
+    /**
      * This is a low-level function.  Normally, 99 percent of text rendering should
      * be down using drawLayout() instead.
      * <p>
@@ -333,25 +345,25 @@ public class Drawable extends GObject
      * @param x
      * @param y
      * @param glyphs
-	 */
+     */
     public void drawGlyphs(GC gc, Font font, int x, int y, GlyphString glyphs) {
-		gdk_draw_glyphs(getHandle(), gc.getHandle(), font.getHandle(), x, y, glyphs.getHandle());
-	}
+        gdk_draw_glyphs(getHandle(), gc.getHandle(), font.getHandle(), x, y, glyphs.getHandle());
+    }
 
-	/**
-         * This is a low-level function.  Normally, 99 percent of text rendering should
-         * be down using drawLayout() instead.
-         * <p>
-         * A glyph is a single image in a font.  This method draws a
-         * sequence of glyphs.
-         * @param font
-         * @param x
-         * @param y
-         * @param glyphs
-	 */
+    /**
+     * This is a low-level function.  Normally, 99 percent of text rendering should
+     * be down using drawLayout() instead.
+     * <p>
+     * A glyph is a single image in a font.  This method draws a
+     * sequence of glyphs.
+     * @param font
+     * @param x
+     * @param y
+     * @param glyphs
+     */
     public void drawGlyphs(Font font, int x, int y, GlyphString glyphs) {
     	drawGlyphs(GC.getDefaultGC(this), font, x, y, glyphs);
-	}
+    }
 
     /**
      * Render a LayoutLine onto this Drawable.  
@@ -360,9 +372,9 @@ public class Drawable extends GObject
      * @param y
      * @param line
      */
-	public void drawLayoutLine(GC gc, int x, int y, LayoutLine line) {
-		gdk_draw_layout_line(getHandle(), gc.getHandle(), x, y, line.getHandle());
-	}
+    public void drawLayoutLine(GC gc, int x, int y, LayoutLine line) {
+        gdk_draw_layout_line(getHandle(), gc.getHandle(), x, y, line.getHandle());
+    }
 
     /**
      * Render a LayoutLine onto this Drawable using the default GC.  
@@ -370,9 +382,9 @@ public class Drawable extends GObject
      * @param y
      * @param line
      */
-	public void drawLayoutLine(int x, int y, LayoutLine line) {
-		drawLayoutLine(GC.getDefaultGC(this), x, y, line);
-	}
+    public void drawLayoutLine(int x, int y, LayoutLine line) {
+        drawLayoutLine(GC.getDefaultGC(this), x, y, line);
+    }
 
     /**
      * Render a Layout onto this Drawable.
@@ -381,9 +393,9 @@ public class Drawable extends GObject
      * @param y
      * @param layout
      */
-	public void drawLayout(GC gc, int x, int y, Layout layout) {
-		gdk_draw_layout(getHandle(), gc.getHandle(), x, y, layout.getHandle());
-	}
+    public void drawLayout(GC gc, int x, int y, Layout layout) {
+        gdk_draw_layout(getHandle(), gc.getHandle(), x, y, layout.getHandle());
+    }
 
     /**
      * Render a Layout onto this Drawable using the default GC.
@@ -391,9 +403,9 @@ public class Drawable extends GObject
      * @param y
      * @param layout
      */
-	public void drawLayout(int x, int y, Layout layout) {
-		drawLayout(GC.getDefaultGC(this), x, y, layout);
-	}
+    public void drawLayout(int x, int y, Layout layout) {
+        drawLayout(GC.getDefaultGC(this), x, y, layout);
+    }
 
     /**
      * Render a LayoutLne onto this Drawable overriding the layout's normal
@@ -405,9 +417,9 @@ public class Drawable extends GObject
      * @param foreground
      * @param background
      */
-	public void drawLayoutLineWithColors(GC gc, int x, int y, LayoutLine line, Color foreground, Color background) {
-		gdk_draw_layout_line_with_colors(getHandle(), gc.getHandle(), x, y, line.getHandle(), foreground.getHandle(), background.getHandle());
-	}
+    public void drawLayoutLineWithColors(GC gc, int x, int y, LayoutLine line, Color foreground, Color background) {
+        gdk_draw_layout_line_with_colors(getHandle(), gc.getHandle(), x, y, line.getHandle(), foreground.getHandle(), background.getHandle());
+    }
 
     /**
      * Render a LayoutLne onto this Drawable overriding the layout's normal
@@ -419,9 +431,9 @@ public class Drawable extends GObject
      * @param foreground
      * @param background
      */
-	public void drawLayoutLineWithColors(int x, int y, LayoutLine line, Color foreground, Color background) {
-		drawLayoutLineWithColors(GC.getDefaultGC(this), x, y, line, foreground, background);
-	}
+    public void drawLayoutLineWithColors(int x, int y, LayoutLine line, Color foreground, Color background) {
+        drawLayoutLineWithColors(GC.getDefaultGC(this), x, y, line, foreground, background);
+    }
 
     /**
      * Render a Layout onto this Drawable overriding the layout's normal
@@ -433,9 +445,9 @@ public class Drawable extends GObject
      * @param foreground
      * @param background
      */
-	public void drawLayoutWithColors(GC gc, int x, int y, Layout layout, Color foreground, Color background) {
-		gdk_draw_layout_with_colors(getHandle(), gc.getHandle(), x, y, layout.getHandle(), foreground.getHandle(), background.getHandle());
-	}
+    public void drawLayoutWithColors(GC gc, int x, int y, Layout layout, Color foreground, Color background) {
+        gdk_draw_layout_with_colors(getHandle(), gc.getHandle(), x, y, layout.getHandle(), foreground.getHandle(), background.getHandle());
+    }
     
     /**
      * Render a Layout onto this Drawable overriding the layout's normal
@@ -447,30 +459,27 @@ public class Drawable extends GObject
      * @param foreground
      * @param background
      */
-	public void drawLayoutWithColors(int x, int y, Layout layout, Color foreground, Color background) {
-		drawLayoutWithColors(GC.getDefaultGC(this), x, y, layout, foreground, background);
-	}
+    public void drawLayoutWithColors(int x, int y, Layout layout, Color foreground, Color background) {
+        drawLayoutWithColors(GC.getDefaultGC(this), x, y, layout, foreground, background);
+    }
     
     /**
      * Get the Display associated with the Drawable.
      */
     public Display getDisplay() {
         Handle hndl = gdk_drawable_get_display(getHandle());
-        GObject obj = getGObjectFromHandle(hndl);
-        if (null != obj)
-            return (Display)obj;
-        return new Display(hndl);
+        if ( hndl != null ) {
+            GObject obj = getGObjectFromHandle(hndl);
+            return (obj != null) ? (Display)obj : new Display(hndl);
+        }
+        return null;
     }
     
     /**
      * Get the Screen associated with the Drawable.
      */
     public Screen getScreen() {
-        Handle hndl = gdk_drawable_get_screen(getHandle());
-        GObject obj = getGObjectFromHandle(hndl);
-        if (null != obj)
-            return (Screen)obj;
-        return new Screen(hndl);
+        return Screen.getScreenFromHandle( gdk_drawable_get_screen(getHandle()) );
     }
 
     /**
@@ -534,9 +543,9 @@ public class Drawable extends GObject
      * @param ydither
      */
     public void drawPixbuf(GC gc, Pixbuf pixbuf, int srcx, int srcy, int destx, int desty, 
-            int width, int height, RgbDither dither, int xdither, int ydither) {
+                           int width, int height, RgbDither dither, int xdither, int ydither) {
         gdk_draw_pixbuf(getHandle(), gc.getHandle(), pixbuf.getHandle(), srcx, srcy, destx, desty,
-                width, height, dither.getValue(), xdither, ydither);
+                        width, height, dither.getValue(), xdither, ydither);
     }
 
     /**
@@ -553,9 +562,9 @@ public class Drawable extends GObject
      * @param ydither
      */
     public void drawPixbuf(Pixbuf pixbuf, int srcx, int srcy, int destx, int desty, 
-            int width, int height, RgbDither dither, int xdither, int ydither) {
+                           int width, int height, RgbDither dither, int xdither, int ydither) {
     	drawPixbuf(GC.getDefaultGC(this), pixbuf, srcx, srcy, destx, desty,
-                width, height, dither, xdither, ydither);
+                   width, height, dither, xdither, ydither);
     }
 
     /**
@@ -625,9 +634,6 @@ public class Drawable extends GObject
         return new Type(gdk_drawable_get_type());
     }
 	
-    /*
-     * BEGINNING OF JNI CODE
-     */
     native static final protected int gdk_drawable_get_type (); // ??? not in the api
     native static final protected Handle gdk_drawable_get_display(Handle drawable);
     native static final protected Handle gdk_drawable_get_screen(Handle drawable);
@@ -666,7 +672,4 @@ public class Drawable extends GObject
     native static final protected void gdk_draw_gray_image (Handle drawable, Handle gc, int x, int y, int width, int height, int dith, byte[] rgbBuf, int rowstride);
     native static final protected void gdk_draw_indexed_image (Handle drawable, Handle gc, int x, int y, int width, int height, int dith, byte[] buf, int rowstride, Handle cmap);
     
-    /*
-     * END OF JNI CODE
-     */
 }
