@@ -8,24 +8,30 @@
  */
 
 package org.gnu.gdk;
-import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
+
+import org.gnu.javagnome.Handle;
+
 /**
  *
  */
-public class EventProperty extends Boxed 
+public class EventProperty extends Event
 {
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
-    native static final protected int getType (Handle obj);
+    /**
+     * Promotion constructor. Construct the specific event object by
+     * promoting (sharing the <tt>Handle</tt> object of) the base
+     * <tt>Event</tt>.
+     */
+    public EventProperty( Event evt ) {
+        // We share the handle with both instances.  This avoids
+        // copying the event (via the constructor ( super( evt ) ).
+        // It is more of a "promotion" constructor.
+        super( evt.getHandle() );
+    }
+
     native static final protected Handle getWindow (Handle obj);
     native static final protected boolean getSendEvent (Handle obj);
     native static final protected int getAtom (Handle obj);
     native static final protected int getTime (Handle obj);
     native static final protected int getState (Handle obj);
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
 }
 
