@@ -8,17 +8,26 @@
  */
 
 package org.gnu.gdk;
-import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
+
+import org.gnu.javagnome.Handle;
+
 /**
  *
  */
-public class EventSelection extends Boxed 
+public class EventSelection extends Event
 {
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
-    native static final protected int getType (Handle obj);
+    /**
+     * Promotion constructor. Construct the specific event object by
+     * promoting (sharing the <tt>Handle</tt> object of) the base
+     * <tt>Event</tt>.
+     */
+    public EventSelection( Event evt ) {
+        // We share the handle with both instances.  This avoids
+        // copying the event (via the constructor ( super( evt ) ).
+        // It is more of a "promotion" constructor.
+        super( evt.getHandle() );
+    }
+
     native static final protected Handle getWindow (Handle obj);
     native static final protected boolean getSendEvent (Handle obj);
     native static final protected int getSelection (Handle obj);
@@ -26,8 +35,5 @@ public class EventSelection extends Boxed
     native static final protected int getProperty (Handle obj);
     native static final protected Handle getRequestor (Handle obj);
     native static final protected int getTime (Handle obj);
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
 }
 
