@@ -8,8 +8,10 @@
  */
 
 package org.gnu.gdk;
+
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
+
 /**
  * All drawing operations in GDK take a GC argument.  A graphics context 
  * encapsulates information about the way things are drawn, such as the
@@ -23,13 +25,30 @@ public class GC extends GObject
 	public GC(Handle handle){
 		super(handle);
 	}
-	
+
+	/**
+	 * @deprecated
+	 * @param window
+	 */
 	public GC(Window window) {
 		super(gdk_gc_new(window.getHandle()));
 	}
+	
+	/**
+	 * Create a new graphics context with default values.
+	 * @param drawable The created GC must always be used with 
+	 * drawables of the same depth as this one.
+	 */
+	public GC(Drawable drawable) {
+		super(gdk_gc_new(drawable.getHandle()));
+	}
 
-
-	public static GC getDefaultGC(org.gnu.gdk.Drawable d){
+	/**
+	 * Convenience method to construct a new GC with
+	 * default values.
+	 * @param d
+	 */
+	public static GC getDefaultGC(Drawable d){
 		return new GC(GC.gdk_gc_new(d.getHandle()));
 	}
 	
@@ -262,17 +281,14 @@ public class GC extends GObject
 	 * if a colormap was set explicitely with setColormap().
 	 */
 	public Colormap getColormap() {
-	    Handle colormapHandle = gdk_gc_get_colormap(getHandle());
-		if (colormapHandle == null)
-			return null;
-		GObject obj = getGObjectFromHandle(colormapHandle);
-		if (null != obj)
-			return (Colormap)obj;
-		return new Colormap(colormapHandle);
+            return Colormap.getColormapFromHandle( gdk_gc_get_colormap(getHandle()) );
 	}
-    
+	
+    /**
+     * Gets the Screen for which this GC was created.
+     */
     public Screen getScreen() {
-        return new Screen(gdk_gc_get_screen(getHandle()));
+        return Screen.getScreenFromHandle(gdk_gc_get_screen(getHandle()));
     }
 
 
