@@ -8,10 +8,11 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.atk.AtkObject;
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
+
 /**
  *
  */
@@ -24,12 +25,7 @@ public class Accessible extends AtkObject
 	
 	public Widget getWidget() {
 		Handle hndl = getWidget(getHandle());
-		if (null == hndl)
-			return null;
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Widget)obj;
-		return new Widget(hndl);
+		return Widget.getWidget(hndl);
 	}
 	
 	/**
