@@ -12,7 +12,7 @@ import java.util.ArrayList;
 import java.util.List;
 
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  * An action which can be triggered by a menu or toolbar item.
@@ -66,6 +66,17 @@ public class Action extends GObject {
 	Action(Handle handle) {
 		super(handle);
 	}
+    
+    static Action getAction(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Action action = (Action) getGObjectFromHandle(handle);
+        if (action == null)
+            action = new Action(handle);
+        
+        return action;
+    }
 	
 	/**
 	 * Returns the name of the action.
@@ -140,11 +151,7 @@ public class Action extends GObject {
 		Handle[] proxies = gtk_action_get_proxies(getHandle());
 		List objs = new ArrayList();
 		for (int i = 0; i < proxies.length; i++) {
-			GObject obj = getGObjectFromHandle(proxies[i]);
-			if (null != obj)
-				objs.add((Widget)obj);
-			else
-				objs.add(new Widget(proxies[i]));
+			objs.add(Widget.getWidget(proxies[i]));
 		}
 		return objs;
 	}
