@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * This container widget wraps a single widget in a frame with a label,
@@ -28,7 +29,7 @@ public class AspectFrame extends Frame {
 	 * @param yAlign The vertical alignment for the child within the allocation
 	 * of the AspectFrame.  This ranges from 0.0 (top aligned) to 1.0 (bottom
 	 * aligned).
-	 * @param ratio, The desired aspect ration.
+	 * @param ratio The desired aspect ration.
 	 * @param obeyChild If true the ratio is ignored and the aspect ratio is
 	 * taken from the requisition of the child.
 	 */
@@ -39,7 +40,7 @@ public class AspectFrame extends Frame {
 	/**
 	 * Construct an AspectFrame using a handle to a native resource.
 	 */
-	public AspectFrame(int handle) {
+	public AspectFrame(Handle handle) {
 	    super(handle);
 	}
 
@@ -51,7 +52,7 @@ public class AspectFrame extends Frame {
 	 * @param yAlign The vertical alignment for the child within the allocation
 	 * of the AspectFrame.  This ranges from 0.0 (top aligned) to 1.0 (bottom
 	 * aligned).
-	 * @param ratio, The desired aspect ration.
+	 * @param ratio The desired aspect ration.
 	 * @param obeyChild If true the ratio is ignored and the aspect ratio is
 	 * taken from the requisition of the child.
 	 */
@@ -66,13 +67,9 @@ public class AspectFrame extends Frame {
 		return new Type(gtk_aspect_frame_get_type());
 	}
 	
-	/****************************************
-	 * BEGINNING OF GENERATED CODE
-	 ****************************************/
+
 	native static final protected int gtk_aspect_frame_get_type();
-	native static final protected int gtk_aspect_frame_new(String label, double xalign, double yalign, double ratio, boolean obeyChild);
-	native static final protected void gtk_aspect_frame_set(int aspect_frame, double xalign, double yalign, double ratio, boolean obeyChild);
-	/****************************************
-	 * END OF GENERATED CODE
-	 ****************************************/
+	native static final protected Handle gtk_aspect_frame_new(String label, double xalign, double yalign, double ratio, boolean obeyChild);
+	native static final protected void gtk_aspect_frame_set(Handle aspect_frame, double xalign, double yalign, double ratio, boolean obeyChild);
+
 }
