@@ -8,9 +8,11 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
+
 /**
  * A CellRenderer is an object used to reder data in a {@link TreeView} widget.
  * This is for all types of Table, Tree and list. Each renderer has one or more
@@ -38,11 +40,23 @@ import org.gnu.glib.Type;
  * <p>For a full overview of the tree and list objects, see the {@link TreeView}
  * description.
  */
-public class CellRenderer extends GObject 
+public class CellRenderer extends GtkObject 
 {
 	CellRenderer(Handle hndl) {
 		super(hndl);
 	}
+    
+    static CellRenderer getCellRenderer(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        CellRenderer obj = (CellRenderer) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null)
+            obj = new CellRenderer(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * fixme:
