@@ -15,10 +15,10 @@ import org.gnu.gdk.Point;
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.EntryEvent;
 import org.gnu.gtk.event.EntryListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * A single line text entry widget. A fairly large set of key bindings are 
@@ -38,6 +38,18 @@ public class Entry extends Widget implements Editable {
 	public Entry(Handle handle) {
 	    super(handle);
 	}
+    
+    public static Entry getEntry(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Entry obj = (Entry) GObject.getGObjectFromHandle(handle);
+        
+        if (obj == null)
+            obj = new Entry(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Sets the text in the widget to the given value, replacing the current 
@@ -77,12 +89,13 @@ public class Entry extends Widget implements Editable {
 	}
 
 	/**
-	 * Sets the character to use in place of the actual text when setVisibility
-	 * has been called to set text visibility to FALSE. i.e. this is the character 
-	 * used in "password mode" to show the user how many characters have been 
-	 * typed. The default invisible char is an asterisk ('*'). If you set the 
-	 * invisible char to 0, then the user will get no feedback at all; there will 
-	 * be no text on the screen as they type.
+	 * Sets the character to use in place of the actual text when
+	 * setVisibility has been called to set text visibility to
+	 * FALSE (ie this is the character used in "password mode" to
+	 * show the user how many characters have been typed). The
+	 * default invisible char is an asterisk ('*'). If you set the
+	 * invisible char to 0, then the user will get no feedback at
+	 * all; there will be no text on the screen as they type.
 	 */
 	public void setInvisibleChar(char character){
 		gtk_entry_set_invisible_char(getHandle(), (byte) character);
@@ -380,10 +393,12 @@ public class Entry extends Widget implements Editable {
 		fireEntryEvent(ee);
 	}
 
-	private void handleInsertText(String text, int length, int position) {
+	private void handleInsertText(String text, int length, Handle position) {
 		EntryEvent ee = new EntryEvent(this, EntryEvent.Type.INSERT_TEXT);
 		ee.setText(text);
-		ee.setInsertPosition(position);
+		//ee.setInsertPosition(position);
+                //Should be something like:
+                ee.setInsertPosition( getIntFromHandle(position) );
 		fireEntryEvent(ee);
 	}
 	
@@ -393,10 +408,7 @@ public class Entry extends Widget implements Editable {
 	
 	public EntryCompletion getCompletion() {
 	    Handle hndl = gtk_entry_get_completion(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (EntryCompletion)obj;
-		return new EntryCompletion(hndl);
+		return EntryCompletion.getEntryCompletion(hndl);
 	}
 
 	
