@@ -9,16 +9,89 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
-import org.gnu.glib.Quark;
+import java.util.List;
+
+import org.gnu.glib.EventMap;
+import org.gnu.glib.GObject;
+import org.gnu.gtk.event.FileChooserEvent;
+import org.gnu.gtk.event.FileChooserListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * This class implements the native methods that are common with both
  * FileChooserDialog and FileChooserWidget.
  */
 class FileChooserHelper {
+    
+    static void fireCurrentFolderChanged(List listeners, FileChooserEvent event) {
+        if (listeners == null)
+            return;
+        
+        for (int i = 0, c = listeners.size(); i < c; ++i) {
+            ((FileChooserListener) listeners.get(i))
+                    .currentFolderChanged(event);
+        }
+    }
+    
+    static void fireFileActivated(List listeners, FileChooserEvent event) {
+        if (listeners == null)
+            return;
+        
+        for (int i = 0, c = listeners.size(); i < c; ++i) {
+            ((FileChooserListener) listeners.get(i))
+                    .fileActivated(event);
+        }
+    }
+    static void fireSelectionChanged(List listeners, FileChooserEvent event) {
+        if (listeners == null)
+            return;
+        
+        for (int i = 0, c = listeners.size(); i < c; ++i) {
+            ((FileChooserListener) listeners.get(i))
+                    .selectionChanged(event);
+        }
+    }
+    
+    static void fireUpdatePreview(List listeners, FileChooserEvent event) {
+        if (listeners == null)
+            return;
+        
+        for (int i = 0, c = listeners.size(); i < c; ++i) {
+            ((FileChooserListener) listeners.get(i))
+                    .updatePreview(event);
+        } 
+    }
+    
+    static void addListener(List listeners, FileChooserListener listener,
+            EventMap eventMap, GObject source) {
+        if (listeners.size() == 0) {
+            FileChooserEvent.Type[] types = FileChooserEvent.Type.getTypes();
+            for (int i = 0; i < types.length; i++) {
+                eventMap.initialize(source, types[i]);
+            }
+        }
+        else if (listeners.contains(listener))
+            return;
+        
+        listeners.add(listener);
+    }
 
-
+    static void removeListener(List listeners, FileChooserListener listener,
+            EventMap eventMap, GObject source) {
+        if (listeners == null) {
+            return;
+        }
+        
+        listeners.remove(listener);
+        
+        if (listeners.size() == 0) {
+            FileChooserEvent.Type[] types = FileChooserEvent.Type.getTypes();
+            for (int i = 0; i < types.length; i++) {                
+                eventMap.uninitialize(source, types[i]);
+            }
+        }
+    }
+    
 	native static final void gtk_file_chooser_set_action(Handle chooser, int action);
 	native static final int gtk_file_chooser_get_action(Handle chooser);
 	native static final void gtk_file_chooser_set_local_only(Handle chooser, boolean localOnly);
@@ -57,15 +130,13 @@ class FileChooserHelper {
 	native static final Handle[] gtk_file_chooser_list_filters(Handle chooser);
 	native static final void gtk_file_chooser_set_filter(Handle chooser, Handle filter);
 	native static final Handle gtk_file_chooser_get_filter(Handle chooser);
-	native static final boolean gtk_file_chooser_add_shortcut_folder(Handle chooser, String folder, int[] error);
-	native static final boolean gtk_file_chooser_remove_shortcut_folder(Handle chooser, String folder, int[] error);
+	native static final boolean gtk_file_chooser_add_shortcut_folder(Handle chooser, String folder, Handle error);
+	native static final boolean gtk_file_chooser_remove_shortcut_folder(Handle chooser, String folder, Handle error);
 	native static final String[] gtk_file_chooser_list_shortcut_folders(Handle chooser);
-	native static final boolean gtk_file_chooser_add_shortcut_folder_uri(Handle chooser, String uri, int[] error);
-	native static final boolean gtk_file_chooser_remove_shortcut_folder_uri(Handle chooser, String uri, int[] error);
+	native static final boolean gtk_file_chooser_add_shortcut_folder_uri(Handle chooser, String uri, Handle error);
+	native static final boolean gtk_file_chooser_remove_shortcut_folder_uri(Handle chooser, String uri, Handle error);
 	native static final String[] gtk_file_chooser_list_shortcut_folder_uris(Handle chooser);
     native static final void gtk_file_chooser_set_show_hidden(Handle chooser, boolean show_hidden);
     native static final boolean gtk_file_chooser_get_show_hidden(Handle chooser);
     native static final private int gtk_file_chooser_get_type();
-    native static final private Handle gtk_file_chooser_error_quark();
-
 }
