@@ -9,10 +9,9 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.gdk.Event;
 import org.gnu.gdk.EventKey;
+import org.gnu.javagnome.Handle;
 
 /**
  * This class is a catch-all for methods in GTK that are not
@@ -150,19 +149,13 @@ public class Gtk
         keySnooperId = 0;
     }
 
-    protected static boolean handleKeySnoopEvent( Handle widget, 
-                                                  Handle event ) {
-        if ( keySnooperMethod != null ) {
-            Widget widg = null;
-            GObject obj = GObject.getGObjectFromHandle( widget );
-            if ( obj == null ) {
-                widg = new Widget( widget );
-            } else {
-                widg = (Widget)obj;
-            }
-            EventKey evt = new EventKey( event );
-            return keySnooperMethod.keyEvent( widg, evt );
-        } else {
+    protected static boolean handleKeySnoopEvent(Handle widget, Handle event) {
+        if (keySnooperMethod != null) {
+            Widget widg = Widget.getWidget(widget);
+            EventKey evt = EventKey.getEventKey(event);
+            return keySnooperMethod.keyEvent(widg, evt);
+        }
+        else {
             return false;
         }
     }
@@ -184,7 +177,7 @@ public class Gtk
 
 	// Load the library
 	static {
-		System.loadLibrary("gtkjni-" + org.gnu.glib.Config.GTK_API_VERSION);
+		System.loadLibrary(org.gnu.glib.Config.LIBRARY_NAME + org.gnu.glib.Config.GTK_API_VERSION);
 	}
 
 
@@ -206,7 +199,6 @@ public class Gtk
     native static final protected int gtk_get_current_event_time ();
     native static final protected Handle gtk_get_event_widget (Handle event);
     native static final protected String gtk_type_name (Handle type);
-    native static final protected Handle gtk_type_from_name (String name);
 
     native static final private boolean gtk_true();
     native static final private boolean gtk_false();
