@@ -9,8 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * HRuler displays a horizontally-oriented scale that optionally can
