@@ -8,9 +8,11 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.gdk.Pixbuf;
 import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
+
 /**
  * This object represents a set of Icons that are a variant of a 
  * particular icon.
@@ -21,7 +23,7 @@ public class IconSet extends Boxed
 	 * Initialize an IconSet with a handle to a native resource.
 	 */
 	public IconSet(Handle handle) {
-		this.handle = handle;
+		super(handle);
 	}
 	
 
@@ -29,14 +31,14 @@ public class IconSet extends Boxed
 	 * Construct a new empty IconSet.
 	 */
 	public IconSet() {
-		handle = gtk_icon_set_new();
+		super(gtk_icon_set_new());
 	}
 	
 	/**
 	 * Construct a new IconSet from a Pixbuf.
 	 */
 	public IconSet(Pixbuf pixbuf) {
-		handle = gtk_icon_set_new_from_pixbuf(pixbuf.getHandle());
+		super(gtk_icon_set_new_from_pixbuf(pixbuf.getHandle()));
 	}
 	
 	/**
@@ -47,7 +49,7 @@ public class IconSet extends Boxed
 	 * @param iconSource The IconSource to add to this IconSet.
 	 */
 	public void addSource(IconSource iconSource) {
-		gtk_icon_set_add_source(handle, iconSource.getHandle());
+		gtk_icon_set_add_source(getHandle(), iconSource.getHandle());
 	}
 	
     public int getType() {
@@ -56,9 +58,8 @@ public class IconSet extends Boxed
 
     native static final protected Handle gtk_icon_set_new ();
     native static final protected Handle gtk_icon_set_new_from_pixbuf (Handle pixbuf);
-    native static final protected Handle gtk_icon_set_ref (Handle iconSet);
+    native static final protected void gtk_icon_set_ref (Handle iconSet);
     native static final protected void gtk_icon_set_unref (Handle iconSet);
-    native static final protected Handle gtk_icon_set_copy (Handle iconSet);
     native static final protected Handle gtk_icon_set_render_icon (Handle iconSet, Handle style, int 
         direction, int state, int size, Handle widget, String detail);
     native static final protected void gtk_icon_set_add_source (Handle iconSet, Handle source);
