@@ -13,10 +13,10 @@ import java.util.Vector;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.MenuItemEvent;
 import org.gnu.gtk.event.MenuItemListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * The MenuItem widget and derived widgets are the only valid children
@@ -46,6 +46,23 @@ public class MenuItem extends Item {
 	public MenuItem(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Create a new MenuItem with a handle to a native resource
+     * returned from a call to the native libraries.
+     * 
+     * @param handle The handle that represents a pointer to a native resource.
+     */
+    public static MenuItem getMenuItem(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        MenuItem obj = (MenuItem) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new MenuItem(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Construct a new menu item with the provided label.
