@@ -12,7 +12,7 @@ import java.util.ArrayList;
 import java.util.List;
 
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  */
