@@ -9,12 +9,12 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
+import org.gnu.glib.MemStruct;
+import org.gnu.javagnome.Handle;
 
 /**
  */
-public class RadioActionEntry extends Boxed {
+public class RadioActionEntry extends MemStruct {
 
 	public RadioActionEntry(
 			String name,
@@ -25,28 +25,17 @@ public class RadioActionEntry extends Boxed {
 			int value) {
 
 		// create the native object
-		handle = allocate();
+		super(allocate());
 		// set the values
-		setName(handle, name);
-		setStockId(handle, stockId);
-		setLabel(handle, label);
-		setAccelerator(handle, accel);
-		setToolTip(handle, tooltip);
-		setValue(handle, value);
+		setName(getHandle(), name);
+		setStockId(getHandle(), stockId);
+		setLabel(getHandle(), label);
+		setAccelerator(getHandle(), accel);
+		setToolTip(getHandle(), tooltip);
+		setValue(getHandle(), value);
 	}
 
-
-	/* (non-Javadoc)
-	 * @see java.lang.Object#finalize()
-	 */
-	protected void finalize() throws Throwable {
-		super.finalize();
-		free(handle);
-	}
-
-
 	native static final protected Handle allocate();
-	native static final protected void free(Handle entry);
 	native static final protected void setName(Handle entry, String name);
 	native static final protected void setStockId(Handle entry, String stockId);
 	native static final protected void setLabel(Handle entry, String label);
