@@ -8,8 +8,10 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
+
 public class Requisition extends Boxed 
 {
 	
@@ -17,9 +19,9 @@ public class Requisition extends Boxed
 	 * Construct a new Requisition.
 	 */
 	public Requisition(int width, int height) {
-		handle = Requisition.gtk_requisition_new();
-		Requisition.setWidth(handle, width);
-		Requisition.setHeight(handle, height);
+		super(Requisition.gtk_requisition_new());
+		Requisition.setWidth(getHandle(), width);
+		Requisition.setHeight(getHandle(), height);
 	}
 	
 	public Requisition(Handle handle) {
@@ -47,7 +49,6 @@ public class Requisition extends Boxed
     native static final protected void setWidth (Handle cptr, int width);
     native static final protected int getHeight (Handle cptr);
     native static final protected void setHeight (Handle cptr, int height);
-    native static final protected Handle gtk_requisition_copy (Handle requisition);
     native static final protected void gtk_requisition_free (Handle requisition);
 
 }
