@@ -8,9 +8,11 @@
  */
 
 package org.gnu.gtk;
+
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
+
 /**
  * Application settings such as theme, fonts, toolbar icon size and cursor blink
  * times. Most of these are set in the gtk theme or gnome control center and so
@@ -21,12 +23,16 @@ import org.gnu.glib.Type;
 public class Settings extends GObject 
 {
 
-	/**
-	 * Retrieve the runtime type used by the GLib library.
-	 */
-	public static Type getType() {
-		return new Type(gtk_settings_get_type());
-	}
+    public Settings() {
+        super( getType() );
+    }
+
+    /**
+     * Retrieve the runtime type used by the GLib library.
+     */
+    public static Type getType() {
+        return new Type(gtk_settings_get_type());
+    }
 
     native static final protected int gtk_settings_get_type ();
     native static final protected Handle gtk_settings_get_default ();
