@@ -12,7 +12,7 @@ package org.gnu.gtk;
 import java.util.ArrayList;
 import java.util.List;
 
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  * This is a widget which contains a list of strings. It is a wrapper class for
@@ -41,7 +41,7 @@ public class SimpleList extends Widget {
 		super(init());
 		dataBlock = new DataColumnString();
 		store = new ListStore(new DataColumn[] { dataBlock });
-		view = new TreeView(getHandle());
+		view = TreeView.getTreeView(getHandle());
 		view.setModel(store);
 		column = new TreeViewColumn();
 		renderer = new CellRendererText();
