@@ -9,8 +9,7 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  * ToolItem is the base class of widgets that can be added to ToolBar objects.
@@ -31,6 +30,20 @@ public class ToolItem extends Bin {
 	ToolItem(Handle hndl) {
 		super(hndl);
 	}
+    
+    /**
+     * Construct a new ToolItem object from a handle to a native widget.
+     */
+    static ToolItem getToolItem(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        ToolItem obj = (ToolItem) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new ToolItem(handle);
+        
+        return obj;
+    }
 	
     /**
      * Create an empty ToolItem.
@@ -242,10 +255,7 @@ public class ToolItem extends Bin {
 	 */
 	public MenuItem retrieveProxyMenuItem() {
 	    Handle hndl = gtk_tool_item_retrieve_proxy_menu_item(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj) 
-			return (MenuItem)obj;
-		return new MenuItem(hndl);
+		return MenuItem.getMenuItem(hndl);
 	}
 	
 	/**
@@ -261,10 +271,7 @@ public class ToolItem extends Bin {
 	 */
 	public MenuItem getProxyMenuItem(String menuItemId) {
 	    Handle hndl = gtk_tool_item_get_proxy_menu_item(getHandle(), menuItemId);
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj) 
-			return (MenuItem)obj;
-		return new MenuItem(hndl);
+		return MenuItem.getMenuItem(hndl);
 	}
 	
 	/**
