@@ -8,25 +8,28 @@
  */
 
 package org.gnu.gtk;
-import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
 
-public class ToolTipsData extends Boxed 
+import org.gnu.glib.MemStruct;
+import org.gnu.javagnome.Handle;
+ 
+public class ToolTipsData extends MemStruct
 {
 	ToolTipsData(Handle handle) {
-		this.handle = handle;
+		super(handle);
 	}
 	
 	public ToolTips getToolTip() {
-		return new ToolTips(getToolTip(handle));
+        Handle handle = getToolTip(getHandle());
+        return ToolTips.getToolTips(handle);
 	}
 	
 	public Widget getWidget() {
-		return new Widget(getWidget(handle));
+        Handle handle = getWidget(getHandle());
+        return Widget.getWidget(handle);
 	}
 	
 	public String getTipText() {
-		return getTipText(handle);
+		return getTipText(getHandle());
 	}
 
     native static final protected Handle getToolTip (Handle cptr);
