@@ -10,8 +10,9 @@
 package org.gnu.gtk;
 
 import java.util.Hashtable;
-import org.gnu.glib.Handle;
+
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * Internal helper class for classes that implement the {@link TreeSortable} 
