@@ -9,6 +9,7 @@
 
 package org.gnu.gtk;
 
+import org.gnu.javagnome.Handle;
 import org.gnu.glib.Type;
 
 /**
@@ -29,7 +30,7 @@ public class VPaned extends Paned {
 	/**
 	 * Construct a new VPaned from a handle to a native resource.
 	 */
-	public VPaned(int handle) {
+	public VPaned(Handle handle) {
 	    super(handle);
 	}
 	
@@ -40,12 +41,6 @@ public class VPaned extends Paned {
 		return new Type(gtk_vpaned_get_type());
 	}
 
-	/****************************************
-	 * BEGINNING OF GENERATED CODE
-	 ****************************************/
 	native static final protected int gtk_vpaned_get_type();
-	native static final protected int gtk_vpaned_new();
-	/****************************************
-	 * END OF GENERATED CODE
-	 ****************************************/
+	native static final protected Handle gtk_vpaned_new();
 }
