@@ -9,7 +9,7 @@
 
 package org.gnu.pango;
 
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  * The AttrString object is used to represent attributes with a string
@@ -21,16 +21,10 @@ public class AttrString extends Attribute
 	/**
 	 */
 	public String getValue(){
-		return getValue(handle);
+		return getValue(getHandle());
 	}
 
 
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
     native static final protected String getValue (Handle obj);
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
 }
 
