@@ -9,18 +9,13 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
 {
 #endif
 
-GdkBitmap * GdkPangoAttrStipple_get_stipple (GdkPangoAttrStipple * cptr) 
-{
-    return cptr->stipple;
-}
-
 /*
  * Class:     org.gnu.gdk.PangoAttrStipple
  * Method:    getStipple
@@ -29,7 +24,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_PangoAttrStipple_getStipple (JNIEnv *
     jobject obj) 
 {
     GdkPangoAttrStipple *obj_g = (GdkPangoAttrStipple *)getPointerFromHandle(env, obj);
-    return getHandleFromPointer(env, GdkPangoAttrStipple_get_stipple (obj_g));
+    return getGObjectHandle(env, obj_g->stipple);
 }
 
 /*
@@ -40,7 +35,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_PangoAttrStipple_gdk_1pango_1attr_1st
     *env, jclass cls, jobject stipple) 
 {
     GdkBitmap *stipple_g = (GdkBitmap *)getPointerFromHandle(env, stipple);
-    return getHandleFromPointer(env, gdk_pango_attr_stipple_new (stipple_g));
+    return getStructHandle(env, gdk_pango_attr_stipple_new (stipple_g), 
+                           NULL, pango_attribute_destroy);
 }
 
 
