@@ -10,7 +10,8 @@
 #include <jni.h>
 #include <string.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include <cairo.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -121,48 +122,34 @@ extern "C"
 //    gdk_pixbuf_render_pixmap_and_mask (pixbuf_g, pixmapReturn_g, maskReturn_g, 
 //            alphaThreshold_g);
 //}
-//
-///*
-// * Class:     org.gnu.gdk.Pixbuf
-// * Method:    gdk_pixbuf_get_from_drawable
-// */
-//JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1drawable (JNIEnv *env, 
-//    jclass cls, jobject dest, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
-//    jint width, jint height) 
-//{
-//    GdkPixbuf *dest_g = (GdkPixbuf *)getPointerFromHandle(env, dest);
-//    GdkDrawable *src_g = (GdkDrawable *)getPointerFromHandle(env, src);
-//    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
-//    gint32 srcX_g = (gint32) srcX;
-//    gint32 srcY_g = (gint32) srcY;
-//    gint32 destX_g = (gint32) destX;
-//    gint32 destY_g = (gint32) destY;
-//    gint32 width_g = (gint32) width;
-//    gint32 height_g = (gint32) height;
-//    return getHandleFromPointer(env, gdk_pixbuf_get_from_drawable (dest_g, src_g, cmap_g, srcX_g, srcY_g, 
-//            destX_g, destY_g, width_g, height_g));
-//}
-//
-///*
-// * Class:     org.gnu.gdk.Pixbuf
-// * Method:    gdk_pixbuf_get_from_image
-// */
-//JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1image (JNIEnv *env, 
-//    jclass cls, jobject dest, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
-//    jint width, jint height) 
-//{
-//    GdkPixbuf *dest_g = (GdkPixbuf *)getPointerFromHandle(env, dest);
-//    GdkImage *src_g = (GdkImage *)getPointerFromHandle(env, src);
-//    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
-//    gint32 srcX_g = (gint32) srcX;
-//    gint32 srcY_g = (gint32) srcY;
-//    gint32 destX_g = (gint32) destX;
-//    gint32 destY_g = (gint32) destY;
-//    gint32 width_g = (gint32) width;
-//    gint32 height_g = (gint32) height;
-//    return getHandleFromPointer(env, gdk_pixbuf_get_from_image (dest_g, src_g, cmap_g, srcX_g, srcY_g, destX_g, 
-//            destY_g, width_g, height_g));
-//}
+
+/*
+ * Class:     org.gnu.gdk.Pixbuf
+ * Method:    gdk_pixbuf_get_from_drawable
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1drawable (JNIEnv *env, 
+    jclass cls, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
+    jint width, jint height) 
+{
+    GdkDrawable *src_g = (GdkDrawable *)getPointerFromHandle(env, src);
+    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
+    return getGObjectHandle(env, gdk_pixbuf_get_from_drawable (NULL, src_g, cmap_g, 
+    		srcX, srcY, destX, destY, width, height));
+}
+
+/*
+ * Class:     org.gnu.gdk.Pixbuf
+ * Method:    gdk_pixbuf_get_from_image
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1image (JNIEnv *env, 
+    jclass cls, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
+    jint width, jint height) 
+{
+    GdkImage *src_g = (GdkImage *)getPointerFromHandle(env, src);
+    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
+    return getGObjectHandle(env, gdk_pixbuf_get_from_image (NULL, src_g, cmap_g, 
+    		srcX, srcY, destX, destY, width, height));
+}
 
 /*
  * Class:     org.gnu.gdk.Pixbuf
@@ -216,11 +203,11 @@ JNIEXPORT jbyteArray JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1pixels (J
     jclass cls, jobject pixbuf) 
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    gchar *result_g = gdk_pixbuf_get_pixels (pixbuf_g);
-    jsize result_len = result_g ? strlen (result_g) : 0;
+    guchar *result_g = gdk_pixbuf_get_pixels (pixbuf_g);
+    jsize result_len = result_g ? strlen ((gchar*)result_g) : 0;
     jbyteArray result_j = result_g ? (*env)->NewByteArray (env, result_len) : NULL;
     if (result_g) 
-    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, result_g);
+    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, (jbyte*)result_g);
     return result_j;
 }
 
@@ -269,7 +256,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new (JNIEnv *env,
     gint32 bitsPerSample_g = (gint32) bitsPerSample;
     gint32 width_g = (gint32) width;
     gint32 height_g = (gint32) height;
-    return getHandleFromPointer(env, gdk_pixbuf_new (colorspace_g, hasAlpha_g, bitsPerSample_g, width_g, 
+    return getGObjectHandle(env, gdk_pixbuf_new (colorspace_g, hasAlpha_g, bitsPerSample_g, width_g, 
             height_g));
 }
 
@@ -280,7 +267,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new (JNIEnv *env,
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1copy (JNIEnv *env, jclass cls, jobject pixbuf) 
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getHandleFromPointer(env, gdk_pixbuf_copy (pixbuf_g));
+    return getGObjectHandle(env, gdk_pixbuf_copy (pixbuf_g));
 }
 
 /*
@@ -295,7 +282,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1subpixbuf (J
     gint32 srcY_g = (gint32) srcY;
     gint32 width_g = (gint32) width;
     gint32 height_g = (gint32) height;
-    return getHandleFromPointer(env, 
+    return getGObjectHandle(env, 
     		gdk_pixbuf_new_subpixbuf (srcPixbuf_g, srcX_g, srcY_g, width_g, height_g));
 }
 
@@ -308,7 +295,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file (
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getHandleFromPointer(env, gdk_pixbuf_new_from_file (filename_g, &error_g));
+    jobject result =  getGObjectHandle(env, gdk_pixbuf_new_from_file (filename_g, &error_g));
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
     if (NULL != error_g)
     	updateHandle(env, error, error_g);
@@ -324,7 +311,23 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file_1
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getHandleFromPointer(env, gdk_pixbuf_new_from_file_at_size (filename_g, width, height, &error_g));
+    jobject result =  getGObjectHandle(env, gdk_pixbuf_new_from_file_at_size (filename_g, width, height, &error_g));
+    (*env)->ReleaseStringUTFChars(env, filename, filename_g);
+    if (NULL != error_g)
+    	updateHandle(env, error, error_g);
+    return result;
+}
+
+/*
+ * Class:     org_gnu_gdk_Pixbuf
+ * Method:    gdk_pixbuf_new_from_file_at_scale
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file_1at_1scale
+  (JNIEnv *env, jclass cls, jstring filename, jint width, jint height, jboolean aspect, jobject error)
+{
+    const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
+    GError* error_g = NULL;
+    jobject result =  getGObjectHandle(env, gdk_pixbuf_new_from_file_at_scale (filename_g, width, height, aspect, &error_g));
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
     if (NULL != error_g)
     	updateHandle(env, error, error_g);
@@ -339,12 +342,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1file_1info
   (JNIEnv *env, jclass cls, jstring filename, jintArray width, jintArray height)
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
-	gint* w = (*env)->GetIntArrayElements(env, width, NULL);
-	gint* h = (*env)->GetIntArrayElements(env, height, NULL);
-	jobject handle = getHandleFromPointer(env, gdk_pixbuf_get_file_info(filename_g, w, h));
-	(*env)->ReleaseIntArrayElements(env, width, (jint*)w, 0);
-	(*env)->ReleaseIntArrayElements(env, height, (jint*)h, 0);
-	return handle;
+    gint* w = (gint*)(*env)->GetIntArrayElements(env, width, NULL);
+    gint* h = (gint*)(*env)->GetIntArrayElements(env, height, NULL);
+    // GdkPixbufFormat is owned by the GdkPixbuf.
+    jobject handle = getStructHandle(env, gdk_pixbuf_get_file_info(filename_g, w, h), NULL, NULL);
+    (*env)->ReleaseIntArrayElements(env, width, (jint*)w, 0);
+    (*env)->ReleaseIntArrayElements(env, height, (jint*)h, 0);
+    return handle;
 }
 
 
@@ -360,7 +364,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1xpm_1d
     const gchar** data_g = (const gchar**)g_malloc(data_len + 1);
     (*env)->GetByteArrayRegion(env, data, 0, data_len, (jbyte*)data_g);
     data_g[data_len] = 0;
-    return getHandleFromPointer(env, gdk_pixbuf_new_from_xpm_data (data_g));
+    return getGObjectHandle(env, gdk_pixbuf_new_from_xpm_data (data_g));
 }
 
 /*
@@ -372,12 +376,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1inline
 {
     gint32 dataLength_g = (gint32) dataLength;
     jint data_len = (*env)->GetArrayLength(env, data);
-    gchar* data_g = (gchar*)g_malloc(data_len + 1);
+    guchar* data_g = (guchar*)g_malloc(data_len + 1);
     gboolean copyPixels_g = (gboolean) copyPixels;
     GError* error_g = NULL;
     (*env)->GetByteArrayRegion(env, data, 0, data_len, (jbyte*)data_g);
     data_g[data_len] = 0;
-    jobject handle = gdk_pixbuf_new_from_inline (dataLength_g, data_g, copyPixels_g, &error_g);
+    jobject handle = getGObjectHandle(env, 
+    	gdk_pixbuf_new_from_inline (dataLength_g, data_g, copyPixels_g, &error_g));
     if (NULL != error_g)
     	updateHandle(env, error, error_g);
     return handle;
@@ -430,6 +435,28 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1save_1to_1buffer
 	return val;
 }
 
+/*
+ * Class:     org_gnu_gdk_Pixbuf
+ * Method:    gdk_pixbuf_rotate_simple
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1rotate_1simple
+  (JNIEnv *env, jclass cls, jobject pixbuf, jint direction)
+{
+    GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
+    return getGObjectHandle(env, gdk_pixbuf_rotate_simple(pixbuf_g, (GdkPixbufRotation)direction));
+}
+
+/*
+ * Class:     org_gnu_gdk_Pixbuf
+ * Method:    gdk_pixbuf_flip
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1flip
+  (JNIEnv *env, jclass cls, jobject pixbuf, jboolean horizontal)
+{
+    GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
+    return getGObjectHandle(env, gdk_pixbuf_flip(pixbuf_g, (gboolean)horizontal));
+}  
+
 /*
  * Class:     org.gnu.gdk.Pixbuf
  * Method:    gdk_pixbuf_fill
@@ -454,7 +481,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1add_1alpha (JNIEn
     gint32 r_g = (gint32) r;
     gint32 g_g = (gint32) g;
     gint32 b_g = (gint32) b;
-    return getHandleFromPointer(env,
+    // returned pixbuf has ref count of 1.  should we use a different getGObjectHandle function here?
+    return getGObjectHandle(env,
     		gdk_pixbuf_add_alpha (pixbuf_g, substituteColor_g, r_g, g_g, b_g));
 }
 
@@ -582,7 +610,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1scale_1simple (JN
     gint32 destWidth_g = (gint32) destWidth;
     gint32 destHeight_g = (gint32) destHeight;
     GdkInterpType interpType_g = (GdkInterpType) interpType;
-    return getHandleFromPointer(env, 
+    return getGObjectHandle(env, 
     		gdk_pixbuf_scale_simple (src_g, destWidth_g, destHeight_g, interpType_g));
 }
 
@@ -602,7 +630,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1composite_1color_
     gint32 checkSize_g = (gint32) checkSize;
     gint32 color1_g = (gint32) color1;
     gint32 color2_g = (gint32) color2;
-    return getHandleFromPointer(env, gdk_pixbuf_composite_color_simple (src_g, destWidth_g, destHeight_g, 
+    return getGObjectHandle(env, gdk_pixbuf_composite_color_simple (src_g, destWidth_g, destHeight_g, 
             interpType_g, overallAlpha_g, checkSize_g, color1_g, color2_g));
 }
 
@@ -622,10 +650,11 @@ JNIEXPORT jbyteArray JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1option (J
     jsize result_len = result_g ? strlen (result_g) : 0;
     jbyteArray result_j = result_g ? (*env)->NewByteArray (env, result_len) : NULL;
     if (result_g) 
-    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, result_g);
+    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, (const jbyte*)result_g);
     return result_j;
 }
 
+
 #ifdef __cplusplus
 }
 
