@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -26,8 +27,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1new (JNIEnv *env,
     GdkWindow *parent_g = (GdkWindow *)getPointerFromHandle(env, parent);
     GdkWindowAttr *attriutes_g = (GdkWindowAttr *)getPointerFromHandle(env, attriutes);
     gint32 attributesMask_g = (gint32) attributesMask;
-    return getHandleFromPointer(env, 
-    	gdk_window_new (parent_g, attriutes_g, attributesMask_g));
+    return getGObjectHandle(env, 
+                            (GObject *)gdk_window_new (parent_g, 
+                                                       attriutes_g, 
+                                                       attributesMask_g));
 }
 
 /*
@@ -56,12 +59,11 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Window_gdk_1window_1destroy (JNIEnv *env
  * Class:     org.gnu.gdk.Window
  * Method:    gdk_window_at_pointer
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1at_1pointer (JNIEnv *env, jclass 
-    cls, jint winX, jint winY) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1at_1pointer (JNIEnv *env, jclass cls) 
 {
-	gint x = (gint)winX;
-	gint y = (gint)winY;
-    return getHandleFromPointer(env, gdk_window_at_pointer(&x, &y));
+    gint x;
+    gint y;
+    return getGObjectHandle(env, (GObject *)gdk_window_at_pointer(&x, &y));
 }
 
 /*
@@ -378,7 +380,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1foreign_1new (JNI
     cls, jint anid) 
 {
     GdkNativeWindow anid_g = (GdkNativeWindow)anid;
-    return getHandleFromPointer(env, gdk_window_foreign_new (anid_g));
+    return getGObjectHandle(env, (GObject *)gdk_window_foreign_new (anid_g));
 }
 
 /*
@@ -617,7 +619,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1get_1parent (JNIE
     cls, jobject window) 
 {
     GdkWindow *window_g = (GdkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gdk_window_get_parent (window_g));
+    return getGObjectHandle(env, (GObject *)gdk_window_get_parent (window_g));
 }
 
 /*
@@ -628,7 +630,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1get_1toplevel (JN
     cls, jobject window) 
 {
     GdkWindow *window_g = (GdkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gdk_window_get_toplevel (window_g));
+    return getGObjectHandle(env, (GObject *)gdk_window_get_toplevel (window_g));
 }
 
 /*
@@ -639,7 +641,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gdk_Window_gdk_1window_1get_1childre
     cls, jobject window) 
 {
     GdkWindow *window_g = (GdkWindow *)getPointerFromHandle(env, window);
-    return getHandleArrayFromGList(env, gdk_window_get_children (window_g));
+    return getGObjectHandlesFromGList(env, gdk_window_get_children (window_g));
 }
 
 /*
@@ -650,7 +652,7 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gdk_Window_gdk_1window_1peek_1childr
     cls, jobject window) 
 {
     GdkWindow *window_g = (GdkWindow *)getPointerFromHandle(env, window);
-    return getHandleArrayFromGList(env, gdk_window_peek_children (window_g));
+    return getGObjectHandlesFromGList(env, gdk_window_peek_children (window_g));
 }
 
 /*
@@ -755,10 +757,13 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Window_gdk_1window_1set_1functions (JNIE
  * Class:     org.gnu.gdk.Window
  * Method:    gdk_window_get_toplevels
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Window_gdk_1window_1get_1toplevels (JNIEnv *env, jclass 
+JNIEXPORT jobjectArray JNICALL Java_org_gnu_gdk_Window_gdk_1window_1get_1toplevels (JNIEnv *env, jclass 
     cls) 
 {
-    return getHandleFromPointer(env, gdk_window_get_toplevels ());
+    GList *list = gdk_window_get_toplevels();
+    jobjectArray ret = getGObjectHandlesFromGList( env, list );
+    g_list_free( list );
+    return ret;
 }
 
 /*
@@ -997,6 +1002,47 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Window_gdk_1window_1get_1internal_1paint
     (*env)->ReleaseIntArrayElements (env, yOffset, (jint *) yOffset_g, 0);
 }
 
+JNIEXPORT jint JNICALL Java_org_gnu_gdk_Window_gdk_1pointer_1grab
+(JNIEnv *env, jclass cls, jobject window, jboolean owner_events, jint event_mask, jobject confine_to, jobject cursor, jint time_)
+{
+    GdkWindow * window_g = (GdkWindow *)getPointerFromHandle(env, window);
+    gboolean owner_events_g = (gboolean)owner_events;
+    GdkEventMask event_mask_g = (GdkEventMask)event_mask;
+    GdkWindow * confine_to_g = (GdkWindow *)getPointerFromHandle(env, confine_to);
+    GdkCursor * cursor_g = (GdkCursor *)getPointerFromHandle(env, cursor);
+    guint32 time__g = (guint32)time_;
+    return (jint)gdk_pointer_grab(window_g, owner_events_g, event_mask_g, confine_to_g, cursor_g, time__g);
+}
+
+JNIEXPORT jint JNICALL Java_org_gnu_gdk_Window_gdk_1keyboard_1grab
+(JNIEnv *env, jclass cls, jobject window, jboolean owner_events, jint time_)
+{
+    GdkWindow * window_g = (GdkWindow *)getPointerFromHandle(env, window);
+    gboolean owner_events_g = (gboolean)owner_events;
+    guint32 time__g = (guint32)time_;
+    return (jint)gdk_keyboard_grab(window_g, owner_events_g, time__g);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Window_gdk_1pointer_1ungrab
+(JNIEnv *env, jclass cls, jint time_)
+{
+    guint32 time__g = (guint32)time_;
+    gdk_pointer_ungrab(time__g);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Window_gdk_1keyboard_1ungrab
+(JNIEnv *env, jclass cls, jint time_)
+{
+    guint32 time__g = (guint32)time_;
+    gdk_keyboard_ungrab(time__g);
+}
+
+JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Window_gdk_1pointer_1is_1grabbed
+(JNIEnv *env, jclass cls)
+{
+    return (jboolean)gdk_pointer_is_grabbed();
+}
+
 
 #ifdef __cplusplus
 }
