@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -33,7 +34,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Bin_gtk_1bin_1get_1child (JNIEnv *env
     bin) 
 {
     GtkBin *bin_g = (GtkBin *)getPointerFromHandle(env, bin);
-    return getHandleFromPointer(env, gtk_bin_get_child (bin_g));
+    return getGObjectHandle(env, (GObject *) gtk_bin_get_child (bin_g));
 }
 
 #ifdef __cplusplus
