@@ -11,14 +11,16 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
 {
 #endif
 	
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TargetEntry_newTargetEntry(JNIEnv *env, jclass cls, jstring name, jint flags, jint id) 
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TargetEntry_newTargetEntry(JNIEnv *env,
+		jclass cls, jstring name, jint flags, jint id) 
 {
 	GtkTargetEntry *targetEntry = g_new(GtkTargetEntry, 1);
 	targetEntry->target = 
@@ -26,7 +28,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TargetEntry_newTargetEntry(JNIEnv *en
 	targetEntry->flags = (guint) flags;
 	targetEntry->info = (guint) id;
 
-	return getHandleFromPointer(env, targetEntry);
+	return getStructHandle(env, targetEntry, NULL, (JGFreeFunc) g_free);
 }
 	
 #ifdef __cplusplus
