@@ -10,7 +10,8 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -34,7 +35,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1type(JNIE
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1new(JNIEnv *env, jclass cls)
 {
-    return getHandleFromPointer(env, gtk_tree_view_new ());
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_new ());
 }
                                                                           
 /*
@@ -45,7 +46,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1new(JNIEnv
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1new_1with_1model(JNIEnv *env, jclass cls, jobject model)
 {
     GtkTreeModel *model_g = (GtkTreeModel *)getPointerFromHandle(env, model);
-    return getHandleFromPointer(env, gtk_tree_view_new_with_model(model_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_new_with_model(model_g));
 }
                                                                           
 /*
@@ -56,7 +57,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1new_1with_1
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1model(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleFromPointer(env, gtk_tree_view_get_model(view_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_get_model(view_g));
 }
                                                                           
 /*
@@ -76,10 +77,12 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1model(JNI
  * Method:    gtk_tree_view_get_selection
  * Signature: (I)I
  */
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1selection(JNIEnv *env, jclass cls, jobject view)
+JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1selection
+		(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleFromPointer(env, gtk_tree_view_get_selection(view_g));
+    GObject *object = (GObject *) gtk_tree_view_get_selection(view_g);
+    return getGObjectHandleAndRef(env, object);
 }
                                                                           
 /*
@@ -90,7 +93,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1select
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1hadjustment(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleFromPointer(env, gtk_tree_view_get_hadjustment(view_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_get_hadjustment(view_g));
 }
                                                                           
 /*
@@ -113,7 +116,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1hadjustme
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1vadjustment(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleFromPointer(env, gtk_tree_view_get_vadjustment(view_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_get_vadjustment(view_g));
 }
                                                                           
 /*
@@ -296,7 +299,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1expander_
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1expander_1column(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleFromPointer(env, gtk_tree_view_get_expander_column(view_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_get_expander_column(view_g));
 }
 
 /*
@@ -484,7 +487,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1cursor(JN
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1bin_1window(JNIEnv *env, jclass cls, jobject view)
 {
     GtkTreeView *view_g = (GtkTreeView *)getPointerFromHandle(env, view);
-    return getHandleFromPointer(env, gtk_tree_view_get_bin_window(view_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_get_bin_window(view_g));
 }
                                                                           
 /*
@@ -751,6 +754,54 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1hover
     return (jboolean)gtk_tree_view_get_hover_expand(tree_view_g);
 }
 
+gboolean static treeViewRowSeparatorFunc( GtkTreeModel *model, 
+                                          GtkTreeIter *iter, 
+                                          gpointer data ) {
+    jobject model_handle;
+    jobject iter_handle;
+	JGFuncCallbackRef *ref = (JGFuncCallbackRef*) data;
+	model_handle = getGObjectHandleAndRef(ref->env, (GObject *) model);
+	iter_handle = getGBoxedHandle(ref->env, iter, GTK_TYPE_TREE_ITER, (GBoxedCopyFunc)
+			gtk_tree_iter_copy, (GBoxedFreeFunc) gtk_tree_iter_free);
+	return (* ref->env)->CallBooleanMethod(ref->env, 
+                                               ref->obj, 
+                                               ref->methodID,
+                                               model_handle,
+                                               iter_handle);
+}
+
+JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1row_1separator_1func
+(JNIEnv *env, jclass cls, jobject tree_view, jobject tview, jstring callback)
+{
+    GtkTreeView * tree_view_g = 
+        (GtkTreeView *)getPointerFromHandle(env, tree_view);
+
+    if ( !tview ) {
+        gtk_tree_view_set_row_separator_func(tree_view_g, NULL, NULL, NULL);
+        return;
+    }
+
+    JGFuncCallbackRef *ref = g_new( JGFuncCallbackRef, 1 );
+    ref->env = env;
+    ref->obj = (* env)->NewGlobalRef(env, tview);
+
+    const char *funcname = (*env)->GetStringUTFChars(env, callback, NULL);
+    // Get method id for the callback method name.
+    ref->methodID = 
+        (*env)->GetMethodID(env, 
+                            (*env)->GetObjectClass(env, ref->obj), 
+                            funcname, "(Lorg/gnu/javagnome/Handle;Lorg/gnu/javagnome/Handle;)Z" );
+    if ( ref->methodID == NULL ) {
+        (*env)->ReleaseStringUTFChars(env, callback, funcname);
+        g_free( ref );
+        // Error!  Throw exception!
+        return;
+    }
+    (*env)->ReleaseStringUTFChars(env, callback, funcname);
+
+    gtk_tree_view_set_row_separator_func(tree_view_g, treeViewRowSeparatorFunc, ref, NULL);
+}
+
 /*
 JNIEXPORT jint JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1row_1separator_1func
 (JNIEnv *env, jclass cls, jobject tree_view)
@@ -759,15 +810,6 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1get_1row_1sepa
     return (jint)gtk_tree_view_get_row_separator_func(tree_view_g);
 }
 
-JNIEXPORT void JNICALL Java_org_gnu_gtk_TreeView_gtk_1tree_1view_1set_1row_1separator_1func
-(JNIEnv *env, jclass cls, jobject tree_view, jint func, gpointer data, jint destroy)
-{
-    GtkTreeView * tree_view_g = (GtkTreeView *)getPointerFromHandle(env, tree_view);
-    GtkTreeViewRowSeparatorFunc func_g = (GtkTreeViewRowSeparatorFunc)func;
-    gpointer data_g = (gpointer)data;
-    GtkDestroyNotify destroy_g = (GtkDestroyNotify)destroy;
-    gtk_tree_view_set_row_separator_func(tree_view_g, func_g, data_g, destroy_g);
-}
 */
 
 #ifdef __cplusplus
